package com.thinkcc.survet.design.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.thinkcc.survet.base.domian.Designer;
import com.thinkcc.survet.base.mapper.DesignerDao;
import com.thinkcc.survet.base.utils.GeneratedValue;
import org.json.XML;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.util.*;
import java.util.concurrent.TimeUnit;

@RestController
public class WxController2 {
    private static final char[] HEX_DIGITS = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
    @Resource
    private RedisTemplate redisTemplate;
    @Autowired
    GeneratedValue generatedValue;
    @Autowired
    DesignerDao designerDao;

    //获取ticket
    @RequestMapping("/wx/qrcode/ticket")
    public String ticket(HttpServletResponse response, HttpServletRequest request) throws IOException {
        InputStream inputStream = null;
        OutputStream os = null;
        try {
            //获取access_token缓存到redis
            String access_token = (String) redisTemplate.opsForValue().get("wx_access_token");
            if (access_token == null) {
                JSONObject jsonData = getJsonData("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=你的微信公众号appid&secret=你的微信公众号secret");
                access_token = jsonData.getString("access_token");
                redisTemplate.opsForValue().set("wx_access_token", access_token, 6900L, TimeUnit.SECONDS);//微信access_token2个小时过期，我们redis要提前5分钟过期避免影响接口访问
            }

            //获取ticket，临时二维码生成必须要ticket
            String url = "https://api.weixin.qq.com/cgi-bin/qrcode/create?access_token=" + access_token;
            HashMap<String, Object> paramMap = new HashMap<>();
            paramMap.put("expire_seconds", 60);
            paramMap.put("action_name", "QR_SCENE");
            paramMap.put("action_info", new HashMap<String, Object>() {{
                put("scene", new HashMap<String, Object>() {{
                    put("scene_id", 123);
                }});
            }});

            JSONObject jsonObject = sendPostByParam(url, paramMap);
            String ticket = jsonObject.getString("ticket");
            return ticket;

        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;

    }

    //创建临时二维码
    @RequestMapping("/wx/qrcode/create")
    public void createQrcode(HttpServletResponse response, HttpServletRequest request, String ticket) throws IOException {
        try {
            getInputStream("https://mp.weixin.qq.com/cgi-bin/showqrcode?ticket=" + URLEncoder.encode(ticket), response);

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    //微信扫二维码回调，需要在微信公众号后台配置
    @RequestMapping("/wx/qrcode/callBack")
    public String callBack(HttpServletResponse response, HttpServletRequest request, String signature, String timestamp, String nonce, String echostr) throws IOException {
        String result = checkSign(signature, timestamp, nonce, echostr);
        if ("FAIL".equals(result)) {//签名认证，防止恶意访问
            return null;
        }
        try {
            InputStream inputStream;
            StringBuffer sb = new StringBuffer();
            inputStream = request.getInputStream();
            String s = null;
            BufferedReader in = new BufferedReader(new InputStreamReader(inputStream, "UTF-8"));
            while ((s = in.readLine()) != null) {
                sb.append(s);
            }
            in.close();
            inputStream.close();
            org.json.JSONObject jsonObject = XML.toJSONObject(sb.toString());

            String msgType = jsonObject.getJSONObject("xml").getString("MsgType");
            String event = jsonObject.getJSONObject("xml").getString("Event");
            if ("event".equals(msgType) && "SCAN".equals(event)) {//用户扫码已关注登录
                String openid = jsonObject.getJSONObject("xml").getString("FromUserName");
                String ticket = jsonObject.getJSONObject("xml").getString("Ticket");
                Designer designer = designerDao.findByOpenId(openid);
                if (designer != null) {//登录成功
                    redisTemplate.opsForValue().set(ticket, openid, 60L, TimeUnit.SECONDS);
                }

            }
            if ("event".equals(msgType) && "subscribe".equals(event)) {//用户扫码未关注登录
                String openid = jsonObject.getJSONObject("xml").getString("FromUserName");
                String ticket = jsonObject.getJSONObject("xml").getString("Ticket");
                Designer designer = designerDao.findByOpenId(openid);
                if (designer == null) {
                    String getUserInfo = "https://api.weixin.qq.com/cgi-bin/user/info?access_token=" + redisTemplate.opsForValue().get("wx_access_token") + "&openid=" + openid + "&lang=zh_CN";
                    JSONObject userInfo = getJsonData(getUserInfo);
                    String nickname = userInfo.getString("nickname");
                    String headimgurl = userInfo.getString("headimgurl");
                    designer = new Designer();
                    designer.setNickName(nickname);
                    designer.setLogo(headimgurl);
                    designer.setOpenId(openid);
                    //这里参考你系统注册时的代码，其中Designer是用户表数据，改成你的用户实体类
                }
                redisTemplate.opsForValue().set(ticket, openid, 60L, TimeUnit.SECONDS);


            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;


    }

    /**
     * token认证
     *
     * @return the formatted bytes.
     */
    private static String checkSign(String signature, String timestamp, String nonce, String echostr) {
        try {
            String[] arr = new String[]{"这里改成你微信公众号配置的Token", timestamp, nonce};
            Arrays.sort(arr);
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < arr.length; i++) {
                sb.append(arr[i]);
            }
            MessageDigest messageDigest = MessageDigest.getInstance("SHA1");
            messageDigest.update(sb.toString().getBytes());
            byte[] bytes = messageDigest.digest();
            int len = bytes.length;
            StringBuilder buf = new StringBuilder(len * 2);
            // 把密文转换成十六进制的字符串形式
            for (int j = 0; j < len; j++) {
                buf.append(HEX_DIGITS[(bytes[j] >> 4) & 0x0f]);
                buf.append(HEX_DIGITS[bytes[j] & 0x0f]);
            }

            if (buf.toString().equals(signature)) {
                return echostr;

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "FAIL";


    }

    public static JSONObject getJsonData(String apiurl) throws Exception {
        Map<String, Object> resultMap = new HashMap<>();
        URL url = new URL(apiurl);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setConnectTimeout(5000);
        connection.setRequestMethod("GET");
        connection.connect();
        if (connection.getResponseCode() != 200) {
            resultMap.put("status", false);
            throw new Exception("网络连接异常");
        }
        InputStream inputStream = connection.getInputStream();
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
        String readLine;
        StringBuffer buffer = new StringBuffer();
        while ((readLine = reader.readLine()) != null) {
            buffer.append(readLine);
        }
        JSONObject jsonObject = JSON.parseObject(buffer.toString());
        inputStream.close();
        return jsonObject;
    }

    public static JSONObject sendPostByParam(String apiurl, Map<String, Object> param) throws Exception {
        Map<String, Object> resultMap = new HashMap<>();
        URL url = new URL(apiurl);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setConnectTimeout(5000);
        connection.setRequestMethod("POST");
        connection.setDoOutput(true);
        connection.setUseCaches(false);

        OutputStream outputStream = connection.getOutputStream();
        outputStream.write(getParamByte(param));
        outputStream.flush();
        outputStream.close();

        connection.connect();
        if (connection.getResponseCode() != 200) {
            resultMap.put("status", false);
            throw new Exception("网络连接异常");
        }

        InputStream inputStream = connection.getInputStream();
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
        String readLine;
        StringBuffer buffer = new StringBuffer();
        while ((readLine = reader.readLine()) != null) {
            buffer.append(readLine);
        }
        JSONObject jsonObject = JSON.parseObject(buffer.toString());
        inputStream.close();
        return jsonObject;
    }

    private static byte[] getParamByte(Map<String, Object> param) {
        String json = JSONObject.toJSONString(param);
        return json.getBytes();
    }

    public static void getInputStream(String apiurl, HttpServletResponse response) throws Exception {
        OutputStream os = null;
        InputStream in = null;
        try {
            Map<String, Object> resultMap = new HashMap<>();
            URL url = new URL(apiurl);
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setConnectTimeout(5000);
            connection.setRequestMethod("GET");
            connection.connect();
            if (connection.getResponseCode() != 200) {
                resultMap.put("status", false);
                throw new Exception("网络连接异常");
            }
            in = connection.getInputStream();
            //输出
            response.reset();
            response.setCharacterEncoding("UTF-8");
            response.setContentType("image/jpeg;charset=utf-8");
            //new String 里边的是为了防止乱码
            //response.setHeader("Content-Disposition","attachment;filename=\""+new String("gggg.jpg".getBytes("ISO8859-1"),"UTF-8")+"\"");
            os = response.getOutputStream();
            byte[] bt = new byte[1024];
            int len = 0;
            while ((len = in.read(bt)) != -1) {
                os.write(bt, 0, len);
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (os != null) {
                os.flush();
                os.close();
            }
            if (in != null) {
                in.close();
            }
        }
    }
}
